@@ -26,6 +26,8 @@ if TYPE_CHECKING:
     ImportType = Union[ast.Import, ast.ImportFrom]
     Flake8Generator = Generator[tuple[int, int, str, Any], None, None]
 
+ATTRIBUTE_PROPERTY = 'flake8-type-checking_parent'
+
 
 class ImportVisitor(ast.NodeTransformer):
     """Map all imports outside of type-checking blocks."""
@@ -99,7 +101,19 @@ class ImportVisitor(ast.NodeTransformer):
     def visit_If(self, node: ast.If) -> Any:
         """Look for a TYPE_CHECKING block."""
         if hasattr(node.test, 'id') and node.test.id == 'TYPE_CHECKING':  # type: ignore
-            self.type_checking_blocks.append((node.lineno, node.end_lineno or node.lineno, node.col_offset))
+            # Here we want to define the line-number-range where the type-checking block exists
+            # Initially I just set the node.lineno and node.end_lineno, but it turns out that else blocks are
+            # included in this span. Because of this, we now first look for else block to help us limit the range
+            start_of_else_block = None
+            if hasattr(node, 'orelse') and node.orelse:
+                # Just set the lineno of the first element in the else block - 1
+                start_of_else_block = node.orelse[0].lineno - 1
+
+            # Define range
+            self.type_checking_blocks.append(
+                (node.lineno, start_of_else_block or node.end_lineno or node.lineno, node.col_offset)
+            )
+
         self.generic_visit(node)
         return node
 
@@ -214,8 +228,8 @@ class ImportVisitor(ast.NodeTransformer):
         """Map names."""
         if self._in_type_checking_block(node):
             return node
-        if hasattr(node, 'parent'):
-            self.uses[f'{node.id}.{node.parent}'] = node  # type: ignore
+        if hasattr(node, ATTRIBUTE_PROPERTY):
+            self.uses[f'{node.id}.{getattr(node, ATTRIBUTE_PROPERTY)}'] = node  # type: ignore
         self.uses[node.id] = node
         return node
 
@@ -254,7 +268,7 @@ class ImportVisitor(ast.NodeTransformer):
         elif isinstance(node, ast.BinOp):
             return
 
-    def _set_child_node_attribute(self, node, attr: str, val: Any):
+    def _set_child_node_attribute(self, node: Any, attr: str, val: Any) -> Any:
         # Set the parent attribute on the current node children
         for key, value in node.__dict__.items():
             if type(value) not in [int, str, list, bool]:
@@ -262,13 +276,19 @@ class ImportVisitor(ast.NodeTransformer):
         return node
 
     def visit_Attribute(self, node: ast.Attribute) -> Any:
+        """
+        Alter attributes slightly so they're more comparable to imports.
+
+        Specifically, we set a property on child attributes so that attributes with the
+        property can have their names altered.
+        """
         # Set the attribute of the current node
         # This lets us read attribute names for `a.b.c` as `a.b.c` when we're treating the
         # `c` node, which is important to match attributes to imports
         with suppress(Exception):
-            parent = node.parent  # type: ignore
+            parent = getattr(node, ATTRIBUTE_PROPERTY)  # type: ignore
             node.attr = f'{node.attr}.{parent}'
-        node = self._set_child_node_attribute(node, 'parent', node.attr)
+        node = self._set_child_node_attribute(node, ATTRIBUTE_PROPERTY, node.attr)
         self.generic_visit(node)
         return node
 
@@ -350,21 +370,19 @@ class TypingOnlyImportsChecker:
         """TC001."""
         for name in set(self.visitor.import_names) - self.visitor.names:
             unused_import, local_import = self.visitor.import_names[name]
-            if local_import:
-                if not any(unused_import in str(use) for use in self.visitor.uses):
-                    obj = self.visitor.local_imports.pop(unused_import)
-                    error_message, node = obj['error'], obj['node']
-                    yield node.lineno, node.col_offset, error_message.format(module=unused_import), None
+            if local_import and all(unused_import not in str(use) for use in self.visitor.uses):
+                obj = self.visitor.local_imports.pop(unused_import)
+                error_message, node = obj['error'], obj['node']
+                yield node.lineno, node.col_offset, error_message.format(module=unused_import), None
 
     def unused_third_party_import(self) -> Flake8Generator:
         """TC002."""
         for name in set(self.visitor.import_names) - self.visitor.names:
             unused_import, local_import = self.visitor.import_names[name]
-            if not local_import:
-                if not any(unused_import in str(use) for use in self.visitor.uses):
-                    obj = self.visitor.remote_imports.pop(unused_import)
-                    error_message, node = obj['error'], obj['node']
-                    yield node.lineno, node.col_offset, error_message.format(module=unused_import), None
+            if not local_import and all(unused_import not in str(use) for use in self.visitor.uses):
+                obj = self.visitor.remote_imports.pop(unused_import)
+                error_message, node = obj['error'], obj['node']
+                yield node.lineno, node.col_offset, error_message.format(module=unused_import), None
 
     def multiple_type_checking_blocks(self) -> Flake8Generator:
         """TC003."""
@@ -380,10 +398,6 @@ class TypingOnlyImportsChecker:
 
                 use = self.visitor.uses[import_name]
 
-                # before we yield we just want to check one thing: whether the use is inside a type checking block..
-                if self.visitor._in_type_checking_block(use):
-                    return
-
                 # .. or whether there is another duplicate import inside the function scope
                 # (if the use is in a function scope)
                 if use.lineno in self.visitor.function_scopes:
@@ -391,8 +405,6 @@ class TypingOnlyImportsChecker:
                         self.visitor.function_scopes[use.lineno]['start'],
                         self.visitor.function_scopes[use.lineno]['end'],
                     ):
-                        print(import_name)
-                        print(self.visitor.function_scopes[i]['imports'])
                         if import_name in self.visitor.function_scopes[i]['imports']:
                             return
 
